  'snake for Picomite LCD 5.07.07

  'play snake game using cursor keys. Avoid eating scrub and yourself
  'eat all fruit to grow.

  'version
  ' s1 = basic ASCII text snake
  ' s2 = using sprites for snake
  ' s3 = adding fruit and tone
  ' s4/s4_1 = 8 sprites
  ' s5/s5_1 = dynamic game experience, music
  ' s6/s6_1 = highscore + compressed music
  ' s7/S7_1 = add sprites to bas file, add intro
  ' snake8 = release for VGA
  ' s8p = release for LCD picomite (ILI9341 320x240)

  'init arrays and restore previous highscore list
  z=255:Dim s(z,1),n$(9):VAR restore

  'start graphics and sound
'  MODE 2
  SetTick 120,music

  'variables used, and default values
  ' s(x,y)=head coord, (v,w)=head motion, tail erase unless food
  ' h=head poiter, t=tail pointer, l=length snake = level
  ' z=max length snake, p=pixel ahead, n$()=highscore list
  ' c=food delay, (i,j)=food coordiates, u=field color dark green
  ' d=tick counter music, e=music pointer, m$=music, f=tone
  ' a$=input, b=key
  h=0:t=0:l=2:d=0:b=131:x=120:y=x:s(h,0)=x:s(h,1)=y:c=79:e=1:u=16384

  'compressed sound score containing bass, metrum, and melody
  m$="QQE\STSE\Q\SOC\X\SC\\\QMA\STSA\Q\QL@\P\Q@\S\QQE\STSE\Q\SOC\X\SC\\\TMA\VXVA\T\VO@\[\XC\\\"

  'create sprite file in filesystem and load 10 sprites
  Blit load 1,"1.bmp",0,0,8,8
  Blit load 2,"2.bmp",0,0,8,8
  Blit load 3,"3.bmp",0,0,8,8
  Blit load 4,"4.bmp",0,0,8,8
  Blit load 5,"5.bmp",0,0,8,8
  Blit load 6,"6.bmp",0,0,8,8
  Blit load 7,"7.bmp",0,0,8,8
  Blit load 8,"8.bmp",0,0,8,8
  Blit load 9,"2r.bmp",0,0,8,8
  Blit load 10,"3r.bmp",0,0,8,8

  'intro screen
  CLS u                              'empty grass field

  'game title
  j=3*Pi/2
  For i=0 To 3*Pi Step Pi/16        'print S as a series of snake sprites
    Blit write 1,80-40*Cos(i),(1+(i>j))*80-40*Sin(i+Pi*(i>j))
    Pause 100
  Next
  Blit write 3,120,160              'append head
  Text 140,40,"nake",,,5,RGB(brown),u

  'legend and enter player name
  For i=4 To 7
    Blit write i,120+i*12,110
  Next
  Blit write 1,168,130:Blit write 8,180,130
  Text 220,110,"good food",,,,,u
  Text 220,130,"death",,,,,u
  Text 168,165,"use cursor keys",,,,,u
  Text 168,200,"enter name  ......",,,,,u
  n$(0)=""
  Do
    a$=Inkey$
    If a$<>"" Then
      n$(0)=n$(0)+a$
      Text 264,200,n$(0),,,,,u
    EndIf
  Loop Until a$=Chr$(13)


  'playfield
  For i=0 To 39                       'bushes around field
    For j=0 To 30
      Blit write 8,8*i,8*j
  Next :Next
  Box 8,8,304,224,1,u,u               'dark green ground


  'main game loop
  Do
    If b>127 Then                     'calculate movement when b<>0
      v=(b=131)-(b=130)               'change only at key press
      w=(b=129)-(b=128)               'else keep moving same
    EndIf

    a$=Inkey$:b=Asc(a$)               'detect arrow keys
    If b=27 Then Save image "game.bmp" 'detect <ESC> to save game

    'movement of snake head and tail
    x=x+8*v:y=y+8*w                   'new x,y no protection relies on fence color
    p=Pixel(x+4,y+4)                  'look ahead what is coming
    Text 0,0,Str$(l)                  'show level = length

    'free move on open terrain
    If p=u Then                       'when background then empty
      Blit write 1,s(h,0),s(h,1)      'write body shape over old head
      h=(h+1) And z:s(h,0)=x:s(h,1)=y 'new head pointer and store x,y
      t=(h-l) And z                   'new tail pointer
      Blit write 2+(v=0)+7*(w+v=1),x,y        'LCD
      'Blit write 2+(v=0),x,y,4+(v=1)+2*(w=1) 'VGA
      Box s(t,0),s(t,1),8,8,1,u,u     'erase tail

    'check for end of game by eating yourself or fence
    'ElseIf p=65280 Or p=RGB(brown) Then  'if wall or body then die, VGA
    ElseIf p=64512 Or p=&hFC8000 Then  'if wall or body then die, LCD
                                       'sprite colors are different from VGA
      SetTick 0,0:Play stop
      Text 160,100,"HIGHSCORE",C,,,,u
      hiscore
      Text 160,60,"You died, <CR> or 'q' to quit",C,,,,u
      Do :a$=Inkey$:Loop While a$=""
      If a$="q" Then End Else Run     'very brute end game control

    'we stumbled upon food. Not categorized, just eat and grow
    Else                              'is food
      Box x,y,8,8,1,u,u               'erase food eaten
      l=Min(l+4,z):c=79               'and (c=79) immediately create new
    EndIf

    'when food eaten, or time elapsed, generate new food
    Inc c,1                           'new food after 80 moves
    If c=80 Then                      'after 80 snake moves provide new food
      Do                              'find free location
        i=8*Int(1+Rnd()*38):j=8*Int(1+Rnd()*28)
      Loop Until Pixel(i+4,j+4)=u
      c=0:Blit write 4+(l>50)+Int(Rnd()*4),i,j    'OK and plot
    EndIf

    'game speed increases when more points are scored
    Pause Max(180-l,80)               'dynamic speed
  Loop


  'dynamic music play
  'beginning of the game only bass isplayed
  'when gaining points more voices are added, and volume increases
Sub music
  gennote
  If f<700 And l>50 Then Play sound 1,B,Q,f,l/16   'melody sounds when l>50

  If (d And 3) = 0 Then
    gennote :Play sound 2,B,Q,f,l/16  'metrum increases
  EndIf

  If d=0 Or d=8 Then
    gennote :Play sound 3,B,S,f       'bass always
  EndIf

  d=(d+1) And 15                      'count in 1/16 note
  If e>Len(m$) Then e=1               'wrap score to beginning
End Sub

Sub gennote
  f=110*2^((Asc(Mid$(m$,e,1))-59)/12) 'calc freq
  Inc e,1                             'next note
End Sub


  'manages the highscore list, uses n$(0) for new entry, show n$(1..9)
Sub hiscore
  n$(0)=Right$("00"+Str$(l),3)+"  "+n$(0)  'combine level and name
  Sort n$():VAR save n$()             'sort and save
  Font 7                              'smaller font to fit list LCD only
  For i=1 To 9
    Text 133,200-i*9,n$(i),,,,,u      'show list
  Next i
  Font 1
End Sub
